from illallangi.rdf.adapters.aviation import AviationAdapter
from illallangi.rdf.adapters.education import EducationAdapter
from illallangi.rdf.adapters.residential import ResidentialAdapter

__all__ = [
    "AviationAdapter",
    "EducationAdapter",
    "ResidentialAdapter",
]



from illallangi.rdf.models.airline import Airline
from illallangi.rdf.models.airport import Airport
from illallangi.rdf.models.course import Course
from illallangi.rdf.models.residence import Residence

__all__ = [
    "Airline",
    "Airport",
    "Course",
    "Residence",
]



class AirlinesNotFoundError(Exception):
    def __init__(
        self,
        airline_iata: list[str],
    ) -> None:
        self.airline_iata = airline_iata
        msg = f"Airline(s) not found: {', '.join(airline_iata)}"
        super().__init__(msg)


class AirlineMixin:
    def get_airlines_query(
        self,
        airline_iata: list[str],
    ) -> str:
        return f"""
    SELECT ?label ?iata ?icao ?alliance WHERE {{
        VALUES (?value) {{ ( "{'" ) ( "'.join([i.upper() for i in airline_iata])}" ) }}
        ?href ip:airlineIataCode ?value.
        ?href rdfs:label ?label .
        ?href ip:airlineIataCode ?iata .
        OPTIONAL {{ ?href ip:airlineIcaoCode ?icao . }}
        OPTIONAL {{
            ?href ip:memberOfAirlineAlliance ?allianceHref .
            ?allianceHref rdfs:label ?alliance .
        }}
        ?href a ic:airline .
    }}
    """

    def get_airlines(
        self,
        *_args: list,
        airline_iata: list[str] | None = None,
        **_kwargs: dict,
    ) -> list[dict]:
        if airline_iata is None:
            return []

        result = self.graph.query(
            self.get_airlines_query(
                airline_iata=airline_iata,
            ),
        )

        result = [
            {str(k): str(b[str(k)]) if str(k) in b else None for k in result.vars}
            for b in result.bindings
        ]

        not_found = [i for i in airline_iata if i not in [j["iata"] for j in result]]
        if not_found:
            raise AirlinesNotFoundError(not_found)

        return result



import diffsync
from partial_date import PartialDate


class Course(diffsync.DiffSyncModel):
    label: str

    country: str
    finish: PartialDate | None
    institution: str
    locality: str
    olc: str
    postal_code: str
    region: str
    start: PartialDate | None
    street: str

    _modelname = "Course"
    _identifiers = ("label",)
    _attributes = (
        "country",
        "finish",
        "institution",
        "locality",
        "olc",
        "postal_code",
        "region",
        "start",
        "street",
    )

    @classmethod
    def create(
        cls,
        adapter: diffsync.Adapter,
        ids: dict,
        attrs: dict,
    ) -> "Course":
        raise NotImplementedError

    def update(
        self,
        attrs: dict,
    ) -> "Course":
        raise NotImplementedError

    def delete(
        self,
    ) -> "Course":
        raise NotImplementedError



from partial_date import PartialDate


class ResidenceMixin:
    def get_residences_query(
        self,
        rdf_root: str,
    ) -> str:
        return f"""
PREFIX i: <http://data.coley.au/rdf/entity#>
PREFIX ip: <http://data.coley.au/rdf/property#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX v: <http://www.w3.org/2006/vcard/ns#>


SELECT ?start ?finish ?label ?street ?locality ?region ?postal_code ?country ?olc WHERE {{
    <{ rdf_root }> ip:residedAt ?residedAt .
    OPTIONAL {{ ?residedAt ip:startTime ?start }} .
    OPTIONAL {{ ?residedAt ip:endTime ?finish }} .
    ?residedAt ip:atResidence ?atResidence .

    OPTIONAL {{ ?atResidence rdfs:label ?label . }}

    OPTIONAL {{ ?atResidence v:Address ?address
        OPTIONAL {{ ?address v:street-address ?street }}
        OPTIONAL {{ ?address v:locality ?locality }}
        OPTIONAL {{ ?address v:region ?region }}
        OPTIONAL {{ ?address v:postal-code ?postal_code }}
        OPTIONAL {{ ?address v:country-name ?country }}
    }}
    OPTIONAL {{ ?atResidence ip:olc ?olc }} .
}}
"""

    def get_residences(
        self,
        *args: list,
        **kwargs: dict,
    ) -> list[dict]:
        result = self.graph.query(
            self.get_residences_query(
                *args,
                **kwargs,
            ),
        )

        return sorted(
            [
                {
                    **{
                        str(k): b[str(k)].value if str(k) in b else None
                        for k in result.vars
                    },
                    "start": PartialDate(b["start"].value)
                    if "start" in b and b["start"].value not in ["Unknown"]
                    else None,
                    "finish": PartialDate(b["finish"].value)
                    if "finish" in b and b["finish"].value not in ["Unknown"]
                    else None,
                }
                for b in result.bindings
            ],
            key=lambda x: str(x["start"]),
        )



from typing import ClassVar

import diffsync

from illallangi.rdf import RDFClient
from illallangi.rdf.models import Residence


class ResidentialAdapter(diffsync.Adapter):
    def __init__(
        self,
        *args: list,
        **kwargs: dict,
    ) -> None:
        super().__init__()
        self.client = RDFClient(
            *args,
            **kwargs,
        )

    Residence = Residence

    top_level: ClassVar = [
        "Residence",
    ]

    type = "rdf_residential"

    def load(
        self,
        *args: list,
        **kwargs: dict,
    ) -> None:
        for obj in self.client.get_residences(
            *args,
            **kwargs,
        ):
            self.add(
                Residence(
                    country=obj["country"],
                    finish=obj["finish"],
                    label=obj["label"],
                    locality=obj["locality"],
                    olc=obj["olc"],
                    postal_code=obj["postal_code"],
                    region=obj["region"],
                    start=obj["start"],
                    street=obj["street"],
                ),
            )



class AirportMixin:
    def get_airports_query(
        self,
        airport_iata: list[str],
    ) -> str:
        return f"""
SELECT ?label ?iata ?icao WHERE {{
    VALUES (?value) {{ ( "{'" ) ( "'.join([i.upper() for i in airport_iata])}" ) }}
    ?href ip:airportIataCode ?value.
    ?href rdfs:label ?label .
    ?href ip:airportIataCode ?iata .
    ?href ip:airportIcaoCode ?icao .
    ?href a ic:airport .
}}
"""

    def get_airports(
        self,
        *_args: list,
        airport_iata: list[str] | None = None,
        **_kwargs: dict,
    ) -> list[dict]:
        if airport_iata is None:
            return []

        result = self.graph.query(
            self.get_airports_query(
                airport_iata=airport_iata,
            ),
        )

        return [
            {str(k): str(b[str(k)]) if str(k) in b else None for k in result.vars}
            for b in result.bindings
        ]
